@@ -45,6 +45,8 @@ from sqlalchemy.orm import backref
 
 import fedmsg.utils
 
+import fmn.lib.defaults
+
 BASE = declarative_base()
 
 log = logging.getLogger(__name__)
@@ -91,7 +93,7 @@ def init(db_url, alembic_ini=None, debug=False, create=False):
 class Context(BASE):
     __tablename__ = 'contexts'
     name = sa.Column(sa.String(50), primary_key=True)
-    description = sa.Column(sa.String(1024), primary_key=True)
+    description = sa.Column(sa.String(1024), unique=True)
     created_on = sa.Column(sa.DateTime, default=datetime.datetime.utcnow)
     detail_name = sa.Column(sa.String(64), nullable=False)
     icon = sa.Column(sa.String(32), nullable=False)
@@ -139,16 +141,22 @@ class Context(BASE):
 
     def _recipients(self, session, config, valid_paths, message):
         """ Returns the list of recipients for a message. """
+
         for user in User.all(session):
             pref = Preference.load(session, user, self)
-            if pref:
-                filter = pref.prefers(session, config, valid_paths, message)
-                if filter:
-                    yield {
-                        'user': user.openid,
-                        pref.context.detail_name: pref.detail_value,
-                        'filter': filter.name,
-                    }
+            if not pref or not pref.detail_values:
+                continue
+
+            flter = pref.prefers(session, config, valid_paths, message)
+            if not flter:
+                continue
+
+            for value in pref.detail_values:
+                yield {
+                    'user': user.openid,
+                    pref.context.detail_name: value.value,
+                    'filter': flter.name,
+                }
 
     def recipients(self, session, config, valid_paths, message):
         return list(self._recipients(session, config, valid_paths, message))
@@ -159,6 +167,8 @@ class User(BASE):
 
     openid = sa.Column(sa.Text, primary_key=True)
     openid_url = sa.Column(sa.Text, unique=True)
+    # doesn't have to be unique, because we'll require the openid url, too.
+    api_key = sa.Column(sa.Text)
     created_on = sa.Column(sa.DateTime, default=datetime.datetime.utcnow)
 
     @classmethod
@@ -171,13 +181,31 @@ class User(BASE):
     def all(cls, session):
         return session.query(cls).all()
 
+    def reset_api_key(self, session):
+        self.api_key = str(uuid.uuid4())
+        session.flush()
+        session.commit()
+
     @classmethod
-    def get_or_create(cls, session, openid, openid_url):
+    def get_or_create(cls, session, openid, openid_url, create_defaults=True):
         user = cls.by_openid(session, openid)
         if not user:
-            user = cls(openid=openid, openid_url=openid_url)
-            session.add(user)
-            session.flush()
+            user = cls.create(session, openid, openid_url, create_defaults)
+        return user
+
+    @classmethod
+    def create(cls, session, openid, openid_url, create_defaults):
+        user = cls(
+            openid=openid,
+            openid_url=openid_url,
+            api_key=str(uuid.uuid4()),
+        )
+        session.add(user)
+        session.flush()
+
+        if create_defaults:
+            fmn.lib.defaults.create_defaults_for(session, user)
+
         return user
 
 
@@ -291,16 +319,16 @@ class Filter(BASE):
         session.commit()
         return filter
 
-    def add_filter(self, session, paths, filt, **kw):
-        if isinstance(filt, basestring):
-            filt = Rule.create_from_code_path(session, paths, filt, **kw)
+    def add_rule(self, session, paths, rule, **kw):
+        if isinstance(rule, basestring):
+            rule = Rule.create_from_code_path(session, paths, rule, **kw)
         elif kw:
             raise ValueError("Cannot handle rule with non-empty kw")
 
-        self.rules.append(filt)
+        self.rules.append(rule)
         session.flush()
         session.commit()
-        return filt
+        return rule
 
     def remove_filter(self, session, code_path, **kw):
         for f in self.rules:
@@ -330,13 +358,43 @@ class Filter(BASE):
         return True
 
 
+class DetailValue(BASE):
+    __tablename__ = 'detail_values'
+    id = sa.Column(sa.Integer, primary_key=True)
+    created_on = sa.Column(sa.DateTime, default=datetime.datetime.utcnow)
+    value = sa.Column(sa.String(1024), unique=True)
+    preference_id = sa.Column(
+        sa.Integer,
+        sa.ForeignKey('preferences.id'))
+    preference = relation('Preference', backref=backref('detail_values'))
+
+    @classmethod
+    def get(cls, session, value):
+        return session.query(cls).filter(cls.value==value).first()
+
+    @classmethod
+    def create(cls, session, value):
+        obj = cls()
+        obj.value = value
+        session.add(obj)
+        session.commit()
+        return obj
+
+    @classmethod
+    def exists(cls, session, value):
+        return (
+            session.query(cls).filter(
+                cls.value == value).count() > 0 or
+            session.query(Confirmation).filter(
+                Confirmation.detail_value == value).count() > 0
+        )
+
+
 class Preference(BASE):
     __tablename__ = 'preferences'
     id = sa.Column(sa.Integer, primary_key=True)
     created_on = sa.Column(sa.DateTime, default=datetime.datetime.utcnow)
 
-    detail_value = sa.Column(sa.String(1024), unique=True)
-
     # Number of seconds that have elapsed since the earliest queued message
     # before we send a digest over whatever medium.
     batch_delta = sa.Column(sa.Integer, nullable=True)
@@ -345,7 +403,9 @@ class Preference(BASE):
     batch_count = sa.Column(sa.Integer, nullable=True)
 
     # Hold the state of start/stop commands to the irc bot and others.
-    enabled = sa.Column(sa.Boolean, default=True, nullable=False)
+    # Disabled by default so that we can provide robust default filters without
+    # forcing new users into an opt-out situation.
+    enabled = sa.Column(sa.Boolean, default=False, nullable=False)
 
     openid = sa.Column(
         sa.Text,
@@ -383,7 +443,7 @@ class Preference(BASE):
         session.commit()
 
     @classmethod
-    def by_user(cls, session, openid, allow_none=False):
+    def by_user(cls, session, openid):
         query = session.query(
             cls
         ).filter(
@@ -392,18 +452,12 @@ class Preference(BASE):
             cls.context_name
         )
 
-        if not allow_none:
-            query = query.filter(sa.not_(cls.detail_value == None))
-
         return query.all()
 
     @classmethod
     def by_detail(cls, session, detail_value):
-        return session.query(
-            cls
-        ).filter(
-            cls.detail_value == detail_value
-        ).first()
+        value = DetailValue.get(session, detail_value)
+        return value.preference
 
     @classmethod
     def create(cls, session, user, context, detail_value=None):
@@ -415,7 +469,11 @@ class Preference(BASE):
         pref.user = user
         pref.context = context
 
-        pref.detail_value = detail_value
+        if detail_value:
+            value = DetailValue.create(session, detail_value)
+            pref.detail_values.append(value)
+
+            session.add(value)
 
         session.add(pref)
         session.flush()
@@ -451,8 +509,10 @@ class Preference(BASE):
             .filter_by(context_name=context)\
             .first()
 
-    def update_details(self, session, value):
-        self.detail_value = value
+    def update_details(self, session, detail_value):
+        detail_value = detail_value.strip()
+        value = DetailValue.create(session, detail_value)
+        self.detail_values.append(value)
         session.flush()
         session.commit()
 
@@ -619,7 +679,7 @@ class Confirmation(BASE):
         # Propagate back to the Preference if everything is good.
         if self.status == 'accepted':
             pref = Preference.load(session, self.openid, self.context_name)
-            pref.detail_value = self.detail_value
+            pref.update_details(session, self.detail_value)
 
         session.flush()
         session.commit()
